//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		angeliris_compiler.cpp
 * @brief		angelscript compiler t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2012, Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see LICENSE
*/
//-----------------------------------------------------------------------
//======================================================================

//======================================================================
// include
#include <angeliris/angeliris_compiler.h>
#include "internal/angeliris_file.h"
#include "internal/angeliris_debug.h"
#include <tchar.h>

namespace 
{
	char* strskipws(char* str)
	{
		if( str != NULL ) 
		{
			while(*str == ' ' || *str == '\t')
			{
				++str;
			}
		}
		return str;
	}
}

namespace angeliris
{

/**********************************************************************//**
 *
 * RXgN^
 *
*//***********************************************************************/
CAsCompiler::CAsCompiler(void)
	: m_pModule(NULL)
{
}

/**********************************************************************//**
 *
 * fXgN^
 *
*//***********************************************************************/
CAsCompiler::~CAsCompiler(void)
{
}

/**********************************************************************//**
 *
 * RXgN^
 *
 -----------------------------------------------------------------------
 * @param [in]	module	= W[
 * @return	
*//***********************************************************************/
bool CAsCompiler::Initialize(asIScriptModule* module)
{
	if( module == NULL ) return false;
	asIScriptEngine* engine = module->GetEngine();
	if( engine == NULL ) return false;
	m_pModule = module;
	return true;
}

/**********************************************************************//**
 *
 * CN[h
 *
 -----------------------------------------------------------------------
 * @param [in]	lpszFileName	= t@CpX
 * @return	
*//***********************************************************************/
int CAsCompiler::Include(const char* lpszFileName)
{
	if( m_pModule == NULL ) return asNO_MODULE;
	detail::CFile file;
	if( !file.Open(lpszFileName, detail::FOPEN_READ) ) return asERROR;
	u32 size = file.GetSize();
	s8* buf = new s8 [size];
	file.Read(buf, size);
	int r = Preprocess(buf, size);
	if( AS_FAILED(r) ) return r;
	r = m_pModule->AddScriptSection(lpszFileName, buf, size);
	delete [] buf;
	return r;
}

/**********************************************************************//**
 *
 * rh
 *
 -----------------------------------------------------------------------
 * @return	
*//***********************************************************************/
int CAsCompiler::Build(void)
{
	ASSERT(m_pModule != NULL);
	int ret = m_pModule->Build();
	return ret;
}

/**********************************************************************//**
 *
 * vvZX
 *
 -----------------------------------------------------------------------
 * @param [in]	buf		= obt@
 * @param [in]	size	= TCY
 * @return	
*//***********************************************************************/
int CAsCompiler::Preprocess(void* buf, u32 size)
{
	ASSERT(m_pModule != NULL);
	char* p = static_cast<char*>(buf);

	if( p == NULL ) return asERROR;

	while(*p != '\0')
	{
		char* top = p;
		// skip whitespace
		p = strskipws(p);

		if( strncmp(p, "#include", 8) == 0 )
		{
			p += 8;
			p = strskipws(p);
			if( *p == '\"' )
			{
				++p;
				char path[MAX_PATH];
				for( int i=0; i < MAX_PATH; ++i )
				{
					path[i] = *p;
					++p;
					if( *p == '\"' || *p == '\n' || *p == '\0' ) break;
				}
				// include
				int r = Include(path);
				if( AS_FAILED(r) ) return r;
				// RgAEg
				*top = '/';
				*(top+1) = '/';
			}
		}

		p = strchr(p, '\n');
		if( p == NULL ) break;
		++p;
	}
	return asSUCCESS;
}

}	// end of namespace angeliris
