//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		angeliris_function.cpp
 * @brief		angelscript ֐IuWFNg t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2012, Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see LICENSE
*/
//-----------------------------------------------------------------------
//======================================================================

//======================================================================
// include
#include <angeliris/angeliris_function.h>
#include <angeliris/angeliris_exception.h>

namespace angeliris
{

/**********************************************************************//**
 *
 * RXgN^
 *
*//***********************************************************************/
CAsFunction::CAsFunction(void)
: m_pFunction(NULL)
{
}

/**********************************************************************//**
 *
 * fXgN^
 *
*//***********************************************************************/
CAsFunction::~CAsFunction(void)
{
	Release();
}

/**********************************************************************//**
 *
 * 
 *
 -----------------------------------------------------------------------
 * @param [in]	system	= VXe
 * @param [in]	module	= W[
 * @param [in]	name	= O
 * @return	
*//***********************************************************************/
bool CAsFunction::Initialize(CAsSystem& system, asIScriptModule* module, LPCSTR name)
{
	Release();
	if( module == NULL ) return false;
	if( name == NULL ) return false;
	asIScriptFunction* func = module->GetFunctionByName(name);
	if( func == NULL )
	{
		func = module->GetFunctionByDecl(name);
	}
	if( func == NULL ) return false;
	if( !system.CreateContext(m_Context) ) return false;
	int ret = m_Context.Prepare(func);
	if( AS_FAILED(ret) ) return false;
	m_pFunction = func;
	return true;
}

/**********************************************************************//**
 *
 * 
 *
*//***********************************************************************/
void CAsFunction::Release(void)
{
	m_Context.Terminate();
	m_pFunction = NULL;
}

/**********************************************************************//**
 *
 * LȃIuWFNgǂ
 *
 -----------------------------------------------------------------------
 * @return	^Ul
*//***********************************************************************/
bool CAsFunction::IsValid(void) const
{
	return m_pFunction != NULL;
}

/**********************************************************************//**
 *
 * ֐̂郂W[擾
 *
 -----------------------------------------------------------------------
 * @return	^Ul
*//***********************************************************************/
asIScriptModule* CAsFunction::GetModule(void) const
{
	if( m_pFunction == NULL ) return NULL;
	asIScriptEngine* engine = m_pFunction->GetEngine();
	if( engine == NULL ) return NULL;
	return engine->GetModule(m_pFunction->GetModuleName());
}

/**********************************************************************//**
 *
 * s
 *
*//***********************************************************************/
void CAsFunction::Execute(void)
{
	int ret = m_Context.Execute();
	if( ret == asEXECUTION_FINISHED )
	{
		const char* p = m_Context.GetExceptionString();
		if( p != NULL )
		{
#if _IRIS_ANGELSCRIPT_HAS_EXCEPTION
			asIScriptEngine* engine = NULL;
			if( m_pFunction != NULL ) engine = m_pFunction->GetEngine();
			int line, column;
			line = m_Context.GetExceptionLineNumber(&column);
			int funcId = m_Context.GetExceptionFunction();
			throw as_exception(p, CAsSystem::GetFunctionName(engine, funcId), line, column);
#endif
		}
	}
}

/**********************************************************************//**
 *
 * ߂l̎擾
 *
 -----------------------------------------------------------------------
 * @return	߂l
*//***********************************************************************/
CAsVariant CAsFunction::GetReturnValue(void) const
{
	if( m_pFunction != NULL ) 
	{
		return m_Context.GetReturnValue(m_pFunction->GetReturnTypeId());
	}
	return CAsVariant();
}

}	// end of namespace angeliris
