//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		angeliris_system.cpp
 * @brief		angelscript VXe t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2012, Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see LICENSE
*/
//-----------------------------------------------------------------------
//======================================================================

//======================================================================
// include
#include <angeliris/angeliris_system.h>
#include <angeliris/angeliris_addon.h>
#include <string>
#include "internal/angeliris_debug.h"

namespace angeliris
{

namespace 
{

/**********************************************************************//**
 *
 * RpCbZ[WR[obN
 *
*//***********************************************************************/
static void OnBuildMessageCallback(const asSMessageInfo *msg, void *param)
{
	const char* type = "ERR ";
	switch( msg->type )
	{
	case asMSGTYPE_ERROR:
		type = "ERROR";
		break;
	case asMSGTYPE_WARNING:
		type = "WARN";
		break;
	case asMSGTYPE_INFORMATION:
		type = "INFO";
		break;
	}

	printf("%s (%d, %d) : %s : %s\n", msg->section, msg->row, msg->col, type, msg->message);
}

// W֐
void asPrint(std::string& msg)
{
	printf(msg.c_str());
}
void asPrintf(std::string& fmt, ...)
{
	va_list va;
	va_start(va, fmt);
	vprintf(fmt.c_str(), va);
	va_end(va);
}
void asPrintf1(std::string& fmt, int n1) { printf(fmt.c_str(), n1); }
void asPrintf2(std::string& fmt, int n1, int n2) { printf(fmt.c_str(), n1, n2); }
void asPrintf3(std::string& fmt, int n1, int n2, int n3) { printf(fmt.c_str(), n1, n2, n3); }
void asPrintf4(std::string& fmt, int n1, int n2, int n3, int n4) { printf(fmt.c_str(), n1, n2, n3, n4); }
void asPrintf5(std::string& fmt, int n1, int n2, int n3, int n4, int n5) { printf(fmt.c_str(), n1, n2, n3, n4, n5); }
void asPrintf6(std::string& fmt, int n1, int n2, int n3, int n4, int n5, int n6) { printf(fmt.c_str(), n1, n2, n3, n4, n5, n6); }
void asPrintf7(std::string& fmt, int n1, int n2, int n3, int n4, int n5, int n6, int n7) { printf(fmt.c_str(), n1, n2, n3, n4, n5, n6, n7); }
void asPrintf8(std::string& fmt, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8) { printf(fmt.c_str(), n1, n2, n3, n4, n5, n6, n7, n8); }
void asPrintf9(std::string& fmt, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int n9) { printf(fmt.c_str(), n1, n2, n3, n4, n5, n6, n7, n8, n9); }
void asPrintf10(std::string& fmt, int n1, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int n9, int n10) { printf(fmt.c_str(), n1, n2, n3, n4, n5, n6, n7, n8, n9, n10); }

}

/**********************************************************************//**
 *
 * RXgN^
 *
*//***********************************************************************/
CAsSystem::CAsSystem(void)
: m_pEngine(NULL)
{
}

/**********************************************************************//**
 *
 * fXgN^
 *
*//***********************************************************************/
CAsSystem::~CAsSystem(void)
{
	Terminate();
}

/**********************************************************************//**
 *
 * VXe
 *
 -----------------------------------------------------------------------
 * @return	
*//***********************************************************************/
bool CAsSystem::Initialize(void)
{
	m_pEngine = asCreateScriptEngine( ANGELSCRIPT_VERSION );
	if( m_pEngine == NULL ) return false;
	m_pEngine->SetMessageCallback(asFUNCTION(OnBuildMessageCallback), NULL, asCALL_CDECL);

	// add_on
	{
		CAsAddOn::Register(m_pEngine);
	}
	// ֐o^
	{
		m_pEngine->RegisterGlobalFunction("void print(string &in)", asFUNCTION(asPrint), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in)", asFUNCTION(asPrint), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int)", asFUNCTION(asPrintf1), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int)", asFUNCTION(asPrintf2), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int)", asFUNCTION(asPrintf3), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int)", asFUNCTION(asPrintf4), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int, int)", asFUNCTION(asPrintf5), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int, int, int)", asFUNCTION(asPrintf6), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int, int, int, int)", asFUNCTION(asPrintf7), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int, int, int, int, int)", asFUNCTION(asPrintf8), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int, int, int, int, int, int)", asFUNCTION(asPrintf9), asCALL_CDECL);
		m_pEngine->RegisterGlobalFunction("void printf(string &in, int, int, int, int, int, int, int, int, int, int)", asFUNCTION(asPrintf10), asCALL_CDECL);
	}
	return true;
}

/**********************************************************************//**
 *
 * I
 *
*//***********************************************************************/
void CAsSystem::Terminate(void)
{
	// context ̏I
	DetachObserverAll();
	if( m_pEngine != NULL )
	{
		m_pEngine->Release();
		m_pEngine = NULL;
	}
}

/**********************************************************************//**
 *
 * ReLXg쐬
 *
 -----------------------------------------------------------------------
 * @param [out]	context	= o
 * @return	
*//***********************************************************************/
bool CAsSystem::CreateContext(CAsContext &context)
{
	context.Terminate();
	ASSERT( m_pEngine != NULL );
	asIScriptContext* p = m_pEngine->CreateContext();
	if( p == NULL ) return false;
	context.m_pContext = p;
	context.SetExceptionCallback(asMETHOD(CAsSystem, ExceptionCallback), this, asCALL_THISCALL);
	AttachObserver(&context);
	return true;
}

/**********************************************************************//**
 *
 * W[̍쐬
 *
 -----------------------------------------------------------------------
 * @param [in]	lpszModuleName	= W[
 * @return	W[
*//***********************************************************************/
asIScriptModule* CAsSystem::CreateModule(LPCSTR lpszModuleName)
{
	ASSERT( m_pEngine != NULL );
	return m_pEngine->GetModule(lpszModuleName, asGM_CREATE_IF_NOT_EXISTS);
}

/**********************************************************************//**
 *
 * W[̎擾
 *
 -----------------------------------------------------------------------
 * @param [in]	lpszModuleName	= W[
 * @return	W[
*//***********************************************************************/
asIScriptModule* CAsSystem::GetModule(LPCSTR lpszModuleName)
{
	ASSERT( m_pEngine != NULL );
	return m_pEngine->GetModule(lpszModuleName);
}

/**********************************************************************//**
 *
 * ֐̎擾
 *
 -----------------------------------------------------------------------
 * @param [in]	funcId	= ֐ID
 * @return	֐
*//***********************************************************************/
LPCSTR CAsSystem::GetFunctionName(int funcId) const
{
	return GetFunctionName(m_pEngine, funcId);
}

/**********************************************************************//**
 *
 * ֐̎擾
 *
 -----------------------------------------------------------------------
 * @param [in]	funcId	= ֐ID
 * @return	֐
*//***********************************************************************/
LPCSTR CAsSystem::GetFunctionName(asIScriptEngine* engine, int funcId)
{
	if( engine == NULL ) return NULL;
	asIScriptFunction* func = engine->GetFunctionById(funcId);
	if( func == NULL ) return NULL;
	return func->GetName();
}

/**********************************************************************//**
 *
 * OR[obN
 *
 -----------------------------------------------------------------------
 * @param [in]	context	= ReLXg
*//***********************************************************************/
void CAsSystem::ExceptionCallback(asIScriptContext* context)
{
	printf("angelscript exception occurred: ");
	int line, column;
	line = context->GetExceptionLineNumber(&column);
	asIScriptFunction* func = context->GetExceptionFunction();
	const char* name = GetFunctionName(func->GetId());
	if( name != NULL )
	{
		printf("%s(%d, %d): ", name, line, column);
	}
	else
	{
		printf("(%d, %d): ", line, column);
	}
	const char* str = context->GetExceptionString();
	if( str != NULL ) printf(str);
}

/**********************************************************************//**
 *
 * ReLXg Detach
 *
 -----------------------------------------------------------------------
 * @param [in]	context	= f^b`ReLXg
*//***********************************************************************/
void CAsSystem::OnDetach(CAsContext* context)
{
	ASSERT( context != NULL );
	context->Terminate();
}

}	// end of namespace angeliris
