//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		angeliris_vm.cpp
 * @brief		angelscript Virtual Machine t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2012, Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see LICENSE
*/
//-----------------------------------------------------------------------
//======================================================================

//======================================================================
// include
#include <angeliris/angeliris_vm.h>
#include <angeliris/angeliris_exception.h>
#include "internal/angeliris_debug.h"

namespace angeliris
{

/**********************************************************************//**
 *
 * RXgN^
 *
*//***********************************************************************/
CAsVM::CAsVM(CAsSystem& system)
: m_rSystem(system)
{
}

/**********************************************************************//**
 *
 * fXgN^
 *
*//***********************************************************************/
CAsVM::~CAsVM(void)
{
	Terminate();
}

/**********************************************************************//**
 *
 * 
 *
 -----------------------------------------------------------------------
 * @param [in]	compiler	= RpC
 * @return	
*//***********************************************************************/
bool CAsVM::Initialize(const CAsCompiler& compiler)
{
	if( m_Context.IsValid() ) return true;	// łɏς

	// W[擾
	asIScriptModule* module = compiler.GetModule();
	if( module == NULL ) return false;
	asIScriptEngine* engine = module->GetEngine();
	if( engine == NULL ) return false;
	if( !m_Module.Initialize(module) ) return false;
	
	// ReLXg쐬
	if( !m_rSystem.CreateContext(m_Context) ) return false;
	return true;
}

/**********************************************************************//**
 *
 * s
 *
 -----------------------------------------------------------------------
 * @param [in]	entry	= Gg[֐
 * @return	
*//***********************************************************************/
int CAsVM::Run(const char* entry)
{
	ASSERT( m_Context.IsValid() );
	// O
	asIScriptFunction* func = m_Module->GetFunctionByName(entry);
	if( func == NULL )
	{
		// `
		func = m_Module->GetFunctionByDecl(entry);
		if( func == NULL )
		{
			return asNO_FUNCTION;
		}
	}

	int ret = m_Context.Prepare(func);
	if( AS_FAILED(ret) ) return ret;

	while(1)
	{
		int ret = m_Context.Execute();

		if( ret == asEXECUTION_FINISHED )
		{
			const char* p = m_Context.GetExceptionString();
			if( p != NULL )
			{
#if _IRIS_ANGELSCRIPT_HAS_EXCEPTION
				int line, column;
				line = m_Context.GetExceptionLineNumber(&column);
				int funcId = m_Context.GetExceptionFunction();
				throw as_exception(p, m_Module.GetFunctionName(funcId), line, column);
#endif
				return asERROR;
			}
			break;
		}
	}
	return asSUCCESS;
}

/**********************************************************************//**
 *
 * I
 *
*//***********************************************************************/
void CAsVM::Terminate(void)
{
	m_Context.Terminate();
	m_Module = NULL;
}

/**********************************************************************//**
 *
 * LȃIuWFNgǂ
 *
 -----------------------------------------------------------------------
 * @param [out]	func	= ֐IuWFNg
 * @return	
*//***********************************************************************/
bool CAsVM::IsValid(void) const
{
	if( !m_Context.IsValid() ) return false;
	return m_Module.is_valid();
}

/**********************************************************************//**
 *
 * Functor ̐ݒ
 *
 -----------------------------------------------------------------------
 * @param [out]	func	= ֐IuWFNg
 * @param [in]	name	= ֐
 * @return	
*//***********************************************************************/
bool CAsVM::MakeFunctor(CAsFunction& func, LPCSTR name) const
{
	if( !IsValid() ) return false;
	if( !func.Initialize(m_rSystem, m_Module, name) ) return false;
	return true;
}

}	// end of namespace iris
